@@ -42,9 +42,10 @@ from oslo_utils import strutils
 from webob import exc
 
 from glance.common import exception
+from glance.common import location_strategy
 from glance.common import timeutils
 from glance.common import wsgi
-from glance.i18n import _, _LE
+from glance.i18n import _, _LE, _LW
 
 CONF = cfg.CONF
 
@@ -712,3 +713,29 @@ def get_stores_from_request(req, body):
     for store in stores:
         glance_store.get_store_from_store_identifier(store)
     return stores
+
+
+def sort_image_locations(locations):
+    if not CONF.enabled_backends:
+        return location_strategy.get_ordered_locations(locations)
+
+    def get_store_weight(location):
+        store_id = location['metadata'].get('store')
+        if not store_id:
+            return 0
+        try:
+            store = glance_store.get_store_from_store_identifier(store_id)
+        except glance_store.exceptions.UnknownScheme:
+            msg = (_LW("Unable to find store '%s', returning "
+                       "default weight '0'") % store_id)
+            LOG.warning(msg)
+            return 0
+        return store.weight if store is not None else 0
+
+    sorted_locations = sorted(locations, key=get_store_weight, reverse=True)
+    scrubbed = []
+    for loc in sorted_locations:
+        scrubbed.append({'store_name': loc["metadata"].get("store")})
+
+    LOG.debug(('Sorted locations: %s'), scrubbed)
+    return sorted_locations
